import TestItem from "./TestItem";


const {ccclass, property} = cc._decorator;

@ccclass
export default class Test extends cc.Component {
    private get sprite_node() : cc.Node {return this.findNode("sprite_node");}
    private get lbl1() : cc.Label {return this.find<cc.Label>("sprite_node/lbl1",cc.Label);}
    private get imgIcon() : cc.Sprite {return this.find<cc.Sprite>("sprite_node/imgIcon",cc.Sprite);}
    private get item() : TestItem {return this.find<TestItem>("item",TestItem);}
    start () {
        this.lbl1.string = "test copy prefab node name";
        this.imgIcon.node.active = false;
        this.sprite_node.opacity = 200;
    }

    /**属性字典 */
    private proDic:Map<string,cc.Component>;
    public constructor() {
        super();
        this.proDic = new Map<string,cc.Component>();
    }
    public onDestroy(): void {
        this.proDic && this.proDic.clear();
        this.proDic = null;
    }

    /**
     * 查找子项，并缓存
     */
    public findNode(path:string):cc.Node {
        let key:string = path;
        if(this.proDic.has(key)){
            return this.proDic.get(key) as any;
        }
        var tempNode:any = cc.find(path,this.node);
        if(tempNode != null){
            this.proDic.set(key,tempNode);
            return tempNode;
        }
        return null;
    }

    /**
     * 查找子项组件，并缓存
     */
    public find<T extends cc.Component>(path:string,type?:any):T {
        let key:string = path+type.name;
        if(this.proDic.has(key)){
            return <T>this.proDic.get(key);
        }
        var tempNode:cc.Node = cc.find(path,this.node);
        if(tempNode != null){
            if(type == null){
                return tempNode as any;
            }
            var component = tempNode.getComponent<T>(type)
            if(component != null){
                this.proDic.set(key,component);
                return component;
            }
        }
        return null;
    }

}
